"""
@file
@brief Defines handlers for a Tornado application.
"""
import logging
import pprint
from tornado.web import RequestHandler
import tornado.web


class _BaseRequestHandler(RequestHandler):
    """
    Base handler. Returns the user.
    """

    def __init__(self, application, request, **kwargs):
        """
        Constructor.
        See `web.py <https://github.com/tornadoweb/tornado/blob/master/tornado/web.py>`_.
        """
        if 'ut_logged' in kwargs:
            # For unit test, we impose a logged user.
            self._logged_user = kwargs['ut_logged']['user']
            del kwargs['ut_logged']
        if 'dbman' in kwargs:
            self._db = kwargs['dbman']
            del kwargs['dbman']
        RequestHandler.__init__(self, application, request, **kwargs)
        self._app_log = logging.getLogger("tornado.application")

    def info(self, message):
        """
        Log information.
        """
        self._app_log.info('[{0}] {1}'.format(
            self.__class__.__name__, message))

    def get_current_user(self):
        """
        Returns the current user.
        """
        if hasattr(self, '_logged_user'):
            res = {'user': self._logged_user}
        else:
            res = self.get_secure_cookie("user")
            if isinstance(res, bytes):
                res = {'user': res.decode('utf-8')}
        self.info('user={0}'.format(res))
        return res


class _TemplateHandler(_BaseRequestHandler):
    """
    Defines the main page handler.
    """

    def __init__(self, application, tmpl_name, request, **kwargs):
        """
        Constructor.
        See `web.py <https://github.com/tornadoweb/tornado/blob/master/tornado/web.py>`_.

        @param  tmpl_name   template name
        @param  context     additional context for the template
        """
        tmpl, torn = self._process_kwargs(kwargs)
        _BaseRequestHandler.__init__(self, application, request, **torn)
        self._tmpl_name = "templates/{0}".format(tmpl_name)
        self._tmpl_context = {'style': '/static/style.css',
                              'icon': '/static/icon.ico',
                              'image': '/static/icon.png'}
        self._tmpl_context.update(tmpl)
        self.info("template='{0}'".format(self._tmpl_name))
        self.info("kwargs={0}".format(torn))
        self.info("context={0}".format(self._tmpl_context))

    def _process_kwargs(self, kwargs):
        """
        Separates attributes templates / tornado.

        @param      kwargs      attributes
        @return                 attributes for temples, attributes for tornado

        An attributes for the template is prefixed with ``'tmpl_'``.
        """
        tmpl = {}
        torn = {}
        for k, v in kwargs.items():
            if k.startswith("tmpl_"):
                tmpl[k[5:]] = v
            else:
                torn[k] = v
        return tmpl, torn

    @tornado.web.authenticated
    def get(self):
        """
        Returns the content page.
        """
        return self.render(self._tmpl_name, **self._tmpl_context)


class LogoutHandler(_TemplateHandler):
    """
    Logout Handler.
    """

    def __init__(self, application, request, **kwargs):
        """
        Expected additional parameters:

        * tmpl_title: application title
        """
        _TemplateHandler.__init__(
            self, application, "unused", request, **kwargs)

    @tornado.web.authenticated
    def get(self):
        """
        Page.
        """
        self.info("logout {0}".format(self.get_current_user()))
        self.clear_cookie("user")
        self.redirect("/")


class LoginHandler(_TemplateHandler):
    """
    Login Handler.
    """

    def __init__(self, application, request, **kwargs):
        """
        Expected additional parameters:

        * title: application title
        """
        users = kwargs.get('users')
        if users is None:
            raise ValueError("No users was defined.\n{0}".format(
                pprint.pformat(kwargs)))
        del kwargs['users']
        _TemplateHandler.__init__(self, application, "login.{0}.html".format(kwargs.get('lang', 'fr')),
                                  request, **kwargs)
        self._users = users
        if not isinstance(users, dict):
            raise TypeError(
                "users must be a dictionary not {0}.".format(type(users)))
        if hasattr(self, '_logged_user'):
            self.set_current_user(self._logged_user)

    @tornado.gen.coroutine
    def get(self):
        """
        Page.
        """
        self.info('processing GET')
        incorrect = self.get_secure_cookie("incorrect")
        if incorrect and int(incorrect) > 20:
            self.write('<center>blocked</center>')
            return
        return self.render(self._tmpl_name, **self._tmpl_context)

    def set_current_user(self, user):
        """
        Sets the current user or clear the cookie if None.
        """
        if user:
            self.info("set user '{0}'".format(user))
            self.set_secure_cookie("user", tornado.escape.json_encode(user))
        else:
            self.clear_cookie("user")

    @tornado.gen.coroutine
    def post(self):
        self.info('processing POST')
        self.info(str(self.request.arguments))
        self.info(str(self.request.headers))
        incorrect = self.get_secure_cookie("incorrect")
        if incorrect and int(incorrect) > 20:
            self.write('<center>blocked</center>')
            return

        getusername = tornado.escape.xhtml_escape(
            self.get_argument("username"))
        getpassword = tornado.escape.xhtml_escape(
            self.get_argument("password"))
        self.info("login='{0}'".format(getusername))
        self.info("nb users={0}".format(len(self._users)))
        if getusername in self._users:
            val = self._users[getusername]
            if isinstance(val, dict):
                goodpwd = val['pwd'] == getpassword
            else:
                goodpwd = self._users[getusername] == getpassword
        if goodpwd:
            self.set_current_user(getusername)
            self.set_secure_cookie("incorrect", "0")
            self.redirect("/")
        else:
            incorrect = self.get_secure_cookie("incorrect") or 0
            increased = str(int(incorrect) + 1)
            self.set_secure_cookie("incorrect", increased)
            self.set_current_user(None)
            self.write("""<center>
                            Something Wrong With Login<br />
                            <a href="/login">Try Again</a>
                          </center>""")


class MainHandler(_TemplateHandler):
    """
    Handlers for the main page.
    """

    def __init__(self, application, request, **kwargs):
        """
        Constructor.
        """
        _TemplateHandler.__init__(
            self, application, "index.{0}.html".format(
                kwargs.get('lang', 'fr')),
            request, **kwargs)


class SubmitForm(_TemplateHandler):
    """
    Handlers for the form to upload dataset.
    """

    def __init__(self, application, request, **kwargs):
        """
        Constructor.
        """
        _TemplateHandler.__init__(
            self, application, "submit.{0}.html".format(
                kwargs.get('lang', 'fr')),
            request, **kwargs)
        val = self.get_argument('cpt_value', None)
        if val is None:
            raise ValueError(
                "cpt_value is not defined in the list of arguments.")
        self._tmpl_context['cpt_value'] = val
        self._tmpl_context['cptidname'] = self._db.get_competitions()


class UploadData(_TemplateHandler):
    """
    Upload data.
    """
    """
    Handlers for the form to upload dataset.
    """

    def __init__(self, application, request, **kwargs):
        """
        Constructor.
        """
        _TemplateHandler.__init__(
            self, application, "waiting.{0}.html".format(
                kwargs.get('lang', 'fr')),
            request, **kwargs)
        self._tmpl_context['waitgif'] = "/static/giphy.gif"

    @tornado.gen.coroutine
    def post(self):
        fileinfo = self.request.files['filearg'][0]
        self.info("fileinfo={0}".format(fileinfo))
        fname = fileinfo['filename']
        self.info("filename='{0}'".format(fname))
        content = fileinfo['body']
        self.info("downloaded size={0}".format(len(content)))


class CompetitionHandler(_TemplateHandler):
    """
    Handlers for the page on competitions.
    """

    def __init__(self, application, request, **kwargs):
        """
        Constructor.
        """
        _TemplateHandler.__init__(
            self, application, "challenge.{0}.html".format(
                kwargs.get('lang', 'fr')),
            request, **kwargs)
        val = self.get_argument('cpt_id', None)
        if val is None:
            raise ValueError(
                "cpt_id is not defined in the list of arguments.")
        self._tmpl_context['cpt'] = self._db.get_competition(val)
        self._tmpl_context['results'] = self._db.get_results(val)



"""
@file
@brief About templates.
"""
import os


def get_template(name):
    """
    Get a template name.

    @param      name    template name
    @return             content
    """
    this = os.path.dirname(name)
    tm = os.path.join(this, name)
    if not os.path.exists(tm):
        raise FileNotFoundError("Unable to find template '{0}'.".format(name))
    with open(tm, "r", encoding="utf-8") as f:
        return f.read()



#-*- coding: utf-8 -*-
"""
@brief      test log(time=1s)
"""

import sys
import os
import unittest
import tornado


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from tornado.testing import AsyncTestCase
from tornado.httpclient import AsyncHTTPClient


class TestHttp(AsyncTestCase):

    @tornado.testing.gen_test
    def test_http_fetch(self):
        client = AsyncHTTPClient(self.io_loop)
        response = yield client.fetch("http://www.xavierdupre.fr")
        self.assertIn(b"Xavier", response.body)


if __name__ == "__main__":
    unittest.main()



"""
@file
@brief Implements metrics.
"""
import sklearn.metrics as metrics
from .classification import roc_auc_score_micro, roc_auc_score_macro, reshape, multi_label_jaccard
from .regression import mse
from .regression_custom import l1_reg_max


def sklearn_metric(met, exp, val):
    """
    Looks into metrics available in
    :epkg:`scikit-learn:metrics`.

    @param      met     function name
    @param      exp     expected values
    @param      val     values
    @return             number
    """
    if isinstance(val, str):
        raise TypeError("val must be a container of floats")
    if isinstance(exp, str):
        raise TypeError("exp must be a container of floats")
    if hasattr(metrics, met):
        f = getattr(metrics, met)
        exp, val = reshape(exp, val)
        return f(exp, val)
    else:
        raise AttributeError("Unable to find metric '{0}'.".format(met))



#-*- coding: utf-8 -*-
"""
@brief      test log(time=1s)
"""
import io
import sys
import os
import unittest
import pandas
import numpy


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import ExtTestCase
from src.lightmlboard.metrics import l1_reg_max, multi_label_jaccard


class TestMetricsCustom(ExtTestCase):

    def test_l1_reg_max(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        exp = [50, 60, 100, 180, 200]
        val = [50, 60, 100, 180, 180]
        r = l1_reg_max(exp, val)
        self.assertEqual(r, 0)

        exp = [50, 60, 100, 180, 200]
        val = [50, 60, 100, 160, 180]
        r = l1_reg_max(exp, val)
        self.assertEqual(r, 0.02222222222222222)

        exp = numpy.array(exp)
        val = numpy.array(val)

        r = l1_reg_max(exp, val)
        self.assertEqual(r, 0.02222222222222222)

        exp = {i: exp[i] for i in range(0, len(exp))}
        val = {i: val[i] for i in range(0, len(val))}
        r = l1_reg_max(exp, val)
        self.assertEqual(r, 0.02222222222222222)

        exp = [50, 60, 100, 180, 200]
        val = [50, 60, 100, 160]
        self.assertRaise(lambda: l1_reg_max(exp, val), ValueError)
        exp = numpy.array(exp)
        val = numpy.array(val)
        self.assertRaise(lambda: l1_reg_max(exp, val), ValueError)
        exp = {i: exp[i] for i in range(0, len(exp))}
        val = {i: val[i] for i in range(0, len(val))}
        self.assertRaise(lambda: l1_reg_max(exp, val), ValueError)
        self.assertRaise(lambda: l1_reg_max(exp, tuple(val)), TypeError)

    def test_l1_reg_max_streams(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        st1 = io.StringIO()
        st2 = io.StringIO()
        exp = [50, 60, 100, 180, 200]
        val = [50, 60, 100, 180, 180]
        d1 = pandas.DataFrame(dict(name=exp)).reset_index(drop=False)
        d2 = pandas.DataFrame(dict(name=val)).reset_index(drop=False)
        d1.to_csv(st1, index=False, header=None, sep=';')
        d2.to_csv(st2, index=False, header=None, sep=';')
        r = l1_reg_max(io.StringIO(st1.getvalue()),
                       io.StringIO(st2.getvalue()))
        self.assertEqual(r, 0)

    def test_classification_jaccard(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        exp = ["4", "5", "6,7", [6, 7], (6, 7), {6, 7}]
        val = ["4", ["5"], "6,7", [6, 7], (6, 7), {6, 7}]
        r = multi_label_jaccard(exp, val)
        self.assertEqual(r, 1)

        exp = ["4", "5", "6,7", [6, 7], (6, 7), {6, 7}]
        val = ["4", ["5"], "7", [7], (7,), {7}]
        r = multi_label_jaccard(exp, val)
        self.assertEqual(r, 0.6666666666666666)

        dexp = {i: exp[i] for i in range(0, len(exp))}
        dval = {i: val[i] for i in range(0, len(val))}
        r = multi_label_jaccard(dexp, dval)
        self.assertEqual(r, 0.6666666666666666)

        val = val[:-1]
        self.assertRaise(lambda: multi_label_jaccard(exp, val), ValueError)
        self.assertRaise(lambda: multi_label_jaccard(
            exp, tuple(val)), TypeError)

    def test_classification_jaccard_streams(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        st1 = io.StringIO()
        st2 = io.StringIO()
        exp = ["4", "5", "6,7", "6,7", "6,7", "6,7"]
        val = ["4", "5", "7", "7", "7", "7,6"]
        d1 = pandas.DataFrame(dict(name=exp)).reset_index(drop=False)
        d2 = pandas.DataFrame(dict(name=val)).reset_index(drop=False)
        d1.to_csv(st1, index=False, header=None, sep=';')
        d2.to_csv(st2, index=False, header=None, sep=';')
        r = multi_label_jaccard(io.StringIO(st1.getvalue()),
                                io.StringIO(st2.getvalue()))
        self.assertEqual(r, 0.75)


if __name__ == "__main__":
    unittest.main()



#-*- coding: utf-8 -*-
import sys
import os
import datetime
import re
import sphinx_rtd_theme


sys.path.insert(0, os.path.abspath(os.path.join(os.path.split(__file__)[0])))
sys.path.insert(
    0,
    os.path.abspath(
        os.path.join(
            os.path.split(__file__)[0],
            "..",
            "..",
            "..",
            "..",
            "pyquickhelper",
            "src")))

local_template = os.path.join(os.path.abspath(
    os.path.dirname(__file__)), "phdoc_templates")

from pyquickhelper.helpgen.default_conf import set_sphinx_variables, get_default_stylesheet
set_sphinx_variables(__file__, "lightmlboard", "Xavier Dupré", 2017,
                     "sphinx_rtd_theme", [
                         sphinx_rtd_theme.get_html_theme_path()],
                     locals(), extlinks=dict(
                         issue=('https://github.com/sdpython/lightmlboard/issues/%s', 'issue')),
                     title="lightmlboard", book=True)

blog_root = "http://www.xavierdupre.fr/app/lightmlboard/helpsphinx/"

html_context = {
    'css_files': get_default_stylesheet() + ['_static/my-styles.css'],
}

html_logo = "project_ico.png"

html_sidebars = {}

language = "en"

mathdef_link_only = True

epkg_dictionary['L1'] = "https://en.wikipedia.org/wiki/Least_absolute_deviations"



# -*- coding: utf-8 -*-
"""
@file
@brief Defines a competition.
"""
import numpy
import pandas
from io import StringIO
from .metrics import mse, sklearn_metric, roc_auc_score_macro, roc_auc_score_micro


class Competition:
    """
    Defines a competition.
    """

    def __init__(self, cpt_id, link, name, description, metric, datafile=None, expected_values=None):
        """
        @param      cpt_id              competition id
        @param      link                link to the page, something like ``/competition``
        @param      name                name of the competition
        @param      metric              metric or list of metrics, list of metrics to compute
        @param      description         description
        @param      datafile            data file
        @param      expected_values     expected values for each metric
        """
        self.link = link
        self.name = name
        self.cpt_id = cpt_id
        if isinstance(metric, str):
            metric = metric.split(',')
        if not isinstance(metric, list):
            metric = [metric]
        self.metrics = metric
        self.datafile = datafile
        self.description = description
        self.expected_values = self._load_values(expected_values)

    def _load_values(self, values):
        """
        Converts values into a list of list of values,
        one per metrics.
        """
        if isinstance(values, str):
            if '\n' in values:
                st = StringIO(values)
                res = pandas.read_csv(st)
            else:
                if self.datafile is None:
                    self.datafile = values
                res = pandas.read_csv(values)
        elif isinstance(values, list):
            if len(values) == 0:
                raise ValueError("values cannot be empty")
            if isinstance(values[0], dict):
                res = pandas.DataFrame(values, header=None, dtype=float)
            else:
                res = pandas.DataFrame(numpy.array(values), dtype=float)
                if res.shape[0] < res.shape[1]:
                    res = res.T.reset_index(drop=True)
            res.columns = ["exp%d" % i for i in range(res.shape[1])]
        elif isinstance(values, pandas.DataFrame):
            res = values
        else:
            raise TypeError(
                "Unexpected type for expected_values: {0}".format(type(values)))
        return res

    def evaluate(self, values):
        """
        Evaluates received values.

        @param      values      list of values
        @return                 dictionary {metric: res}
        """
        res = {}
        values = self._load_values(values)
        for met in self.metrics:
            res[met] = self.evaluate_metric(met, self.expected_values, values)
        return res

    def evaluate_metric(self, met, exp, val):
        """
        Evaluates a metric.

        @param      met     metric
        @param      exp     expected value
        @param      val     values
        @return             result
        """
        if met == "mse":
            return mse(exp, val)
        elif met == "roc_auc_score_micro":
            return roc_auc_score_micro(exp, val)
        elif met == "roc_auc_score_macro":
            return roc_auc_score_macro(exp, val)
        else:
            return sklearn_metric(met, exp, val)

    @property
    def metric(self):
        """
        Returns the metrics in a single string.
        """
        return ",".join(self.metrics)

    def to_dict(self):
        """
        Convert a competition into a dictionary.
        """
        s = StringIO()
        self.expected_values.to_csv(s, index=False)
        val = s.getvalue()
        return dict(cpt_id=self.cpt_id, link=self.link, name=self.name,
                    description=self.description, expected_values=val,
                    metric=",".join(self.metrics), datafile=self.datafile)

    @staticmethod
    def to_records(list_cpt):
        """
        Converts a list of competitions into a list of dictionaries.
        """
        res = []
        for cpt in list_cpt:
            for met in cpt.metrics:
                s = StringIO()
                cpt.expected_values.to_csv(s, index=False)
                val = s.getvalue()
                d = dict(link=cpt.link, cpt_name=cpt.name, metric=met,
                         description=cpt.description, expected_values=val)
                res.append(d)
        return res



"""
@file
@brief Metrics about regressions.
"""
from sklearn.metrics import mean_squared_error


def mse(exp, val):
    """
    Computes `mean_squared_error <http://scikit-learn.org/stable/modules/generated/sklearn.metrics.mean_squared_error.html>`_.
    """
    return mean_squared_error(exp, val)



"""
@file
@brief Defines a Tornado application.
Tutorial `chat <https://github.com/tornadoweb/tornado/tree/stable/demos/chat>`_.
"""
import logging
import os
import pprint
from tornado.web import Application
from tornado.web import StaticFileHandler
from tornado.log import enable_pretty_logging
from .handlersml import MainHandler, LoginHandler, LogoutHandler, UploadData, SubmitForm, CompetitionHandler
from .default_options import LightMLBoardDefaultOptions
from .options_helpers import read_options, read_users
from .dbmanager import DatabaseCompetition


class LightMLBoard(Application):
    """
    Overloads a :epkg:`tornado:Application`.
    """

    def __init__(self, handlers=None, default_host=None, transforms=None,
                 **settings):
        """
        Constructor.
        See `wep.py <https://github.com/tornadoweb/tornado/blob/master/tornado/web.py>`_.
        """
        if "login_url" not in settings:
            settings['login_url'] = "/login"
        if "xsrf_cookies" not in settings:
            settings["xsrf_cookies"] = True

        Application.__init__(self, handlers=handlers, default_host=default_host,
                             transforms=transforms, **settings)
        app_log = logging.getLogger("tornado.application")
        app_log.info('[LightMLBoard] Settings: {0}'.format(settings))

    def update_options(config_options):
        """
        Returns updated options, includes the default ones.
        """
        context = {}
        lang = None
        debug = False
        cookie_secret = None
        enablelog = False
        for opts in [LightMLBoardDefaultOptions.__dict__,
                     config_options]:
            for k, v in opts.items():
                if not k.startswith("_"):
                    if k == 'cookie_secret':
                        cookie_secret = v
                    elif k == "lang":
                        lang = v
                    elif k == "debug":
                        debug = debug
                    elif k == "logging":
                        enablelog = v
                    elif k == "allowed_users":
                        context[k] = v
                    else:
                        context["tmpl_" + k] = v
        app_options = dict(lang=lang, debug=debug, enablelog=enablelog,
                           cookie_secret=cookie_secret)
        return context, app_options

    @staticmethod
    def make_app(config=None, logged=None, dbfile=":memory:"):
        """
        Creates a *LightMLBoard* application.

        @param      config      configuration file
        @param      logged      to log one user
        @param      dbfile      file for the db file
        @return                 @see cl LightMLBoard
        """
        this = os.path.dirname(__file__)
        st = os.path.join(this, "static")

        # Update the context for static files.
        updated_context = {'path': st}

        # Options.
        config_options = read_options(config)
        context, local_context = LightMLBoard.update_options(config_options)
        allowed = context["allowed_users"]
        if allowed is None:
            raise ValueError("No allowed users.\n{0}".format(
                pprint.pformat(context)))
        users = read_users(allowed)

        # db manager
        if config is not None:
            dbman = DatabaseCompetition(dbfile)
            dbman.init_from_options(context)
            context['dbman'] = dbman

        # We remove the users.
        del context['allowed_users']

        # Specifies a logged user.
        if logged:
            context["ut_logged"] = logged
            updated_context["ut_logged"] = logged

        context_users = context.copy()
        context_users['users'] = users

        # Logging.
        if local_context['debug'] or local_context['enablelog']:
            enable_pretty_logging()

        pages = [
            (r"/", MainHandler, context),
            (r'/login', LoginHandler, context_users),
            (r'/logout', LogoutHandler, context),
            (r'/static/(.*)', StaticFileHandler, updated_context),
            (r'/submit', SubmitForm, context),
            (r'/upload', UploadData, context),
            (r'/competition', CompetitionHandler, context),
        ]

        args = dict(lang=local_context["lang"], debug=local_context['debug'])
        if 'cookie_secret' in local_context:
            args['cookie_secret'] = local_context['cookie_secret']
        return LightMLBoard(pages, **args)

    @staticmethod
    def start_app(port=8897, **kwargs):
        """
        Starts the application.

        @param      port        port to listen
        @param      kwargs      @see me make_app
        """
        import tornado.ioloop
        app = LightMLBoard.make_app(**kwargs)
        app.listen(port)
        tornado.ioloop.IOLoop.current().start()



"""
@file
@brief run all unit tests
"""

import os
import sys


def main():
    try:
        import pyquickhelper as skip_
    except ImportError:
        sys.path.append(
            os.path.normpath(
                os.path.abspath(
                    os.path.join(
                        os.path.split(__file__)[0],
                        "..",
                        "..",
                        "pyquickhelper",
                        "src"))))
        if "PYQUICKHELPER" in os.environ and len(os.environ["PYQUICKHELPER"]) > 0:
            sys.path.append(os.environ["PYQUICKHELPER"])
        import pyquickhelper as skip_

    from pyquickhelper.loghelper import fLOG
    from pyquickhelper.pycode import main_wrapper_tests
    fLOG(OutputPrint=True)
    main_wrapper_tests(__file__)


if __name__ == "__main__":
    main()



"""
@file
@brief Metrics about regressions.
"""
import io
import numpy
import pandas


def l1_reg_max(exp, val, max_val=180):
    """
    Implements a :epkg:`L1` scoring function which does not consider
    error above threshold *max_val*.

    @param      exp     list of values or :epkg:`numpy:array`
    @param      val     list of values or :epkg:`numpy:array`
    @return             score

    If ``max_val==180`, the function computes:

    .. math::

        E = \\frac{1}{n} \\sum_{i=1}^n \\frac{\\min \\left| Y_i - \\min(f(X_i), 180) \\right|}{180}

    The computation is faster if :epkg:`numpy:array` are used
    (for *exp* and *val*). *exp and *val* can be filenames or streams.
    In that case, the function expects to find two columns: id, value
    in both files or streams.
    """
    if isinstance(exp, numpy.ndarray) and isinstance(val, numpy.ndarray):
        if len(exp) != len(val):
            raise ValueError(
                "Dimension mismatch {0} != {1}".format(len(exp), len(val)))
        an = numpy.zeros((len(exp),))
        an[:] = max_val
        mv = numpy.minimum(an, val)
        me = numpy.minimum(an, exp)
        df = numpy.abs(mv - me) / max_val
        return df.mean()
    elif isinstance(exp, dict) and isinstance(val, dict):
        if len(exp) != len(val):
            raise ValueError(
                "Dimension mismatch {0} != {1}".format(len(exp), len(val)))
        r = 0.0
        for k, e in exp.items():
            if k in val:
                v = val[k]
                mv = min(v, max_val)
                ev = min(e, max_val)
                d = abs(mv - ev)
                r += 1. * d / max_val
            else:
                raise ValueError("Missing key in prediction {0}".format(k))
        return r / len(exp)
    elif isinstance(exp, (str, io.StringIO)) and isinstance(val, (str, io.StringIO)):
        # We expect filenames.
        d1 = pandas.read_csv(exp, header=None, sep=";")
        d2 = pandas.read_csv(val, header=None, sep=";")
        dd1 = {}
        for k, v in d1.itertuples(name=None, index=False):
            if k in dd1:
                raise KeyError("Key '{}' already present".format(k))
            dd1[k] = v
        dd2 = {}
        for k, v in d2.itertuples(name=None, index=False):
            if k in dd2:
                raise KeyError("Key '{}' already present".format(k))
            dd2[k] = v
        return l1_reg_max(dd1, dd2, max_val=max_val)
    elif isinstance(exp, list) and isinstance(val, list):
        if len(exp) != len(val):
            raise ValueError(
                "Dimension mismatch {0} != {1}".format(len(exp), len(val)))
        r = 0.0
        for e, v in zip(exp, val):
            mv = min(v, max_val)
            ev = min(e, max_val)
            d = abs(mv - ev)
            r += 1. * d / max_val
        return r / len(exp)
    else:
        raise TypeError(
            "Inconsisten types {0} != {1}".format(type(exp), type(val)))



#-*- coding: utf-8 -*-
"""
@brief      test log(time=1s)
"""

import sys
import os
import unittest
import pandas
from io import StringIO


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import ExtTestCase, get_temp_folder
from src.lightmlboard.dbmanager import DatabaseCompetition
from src.lightmlboard.competition import Competition


class TestDb(ExtTestCase):

    def test_creation_memory(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        db = DatabaseCompetition(":memory:")
        db.connect()
        dbl = db.get_table_list()
        db.close()
        self.assertEqual(
            dbl, ['competitions', 'players', 'submissions', 'teams'])

    def test_creation_file(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        temp = get_temp_folder(__file__, "temp_creation_file")
        name = os.path.join(temp, "ex.db3")
        db = DatabaseCompetition(name)
        db.connect()
        dbl = db.get_table_list()
        db.close()
        self.assertEqual(
            dbl, ['competitions', 'players', 'submissions', 'teams'])
        self.assertExists(name)

    def test_creation_db(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        db = DatabaseCompetition(":memory:")
        data = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        opt = os.path.join(data, "ex_default_options.py")
        db.init_from_options(opt)
        db.connect()
        dft = db.to_df("teams")
        dfp = db.to_df("players")
        dfc = db.to_df("competitions")
        dfs = db.to_df("submissions")
        db.close()
        self.assertEqual(dft.shape, (1, 2))
        self.assertEqual(dft.iloc[0, 1], "team1")
        self.assertEqual(dfp.shape, (1, 7))
        self.assertEqual(dfc.shape, (1, 7))
        self.assertEqual(dfs.shape, (1, 7))

    def test_submission(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        data = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        fname = os.path.join(data, "off_eval_all_Y.txt")
        df = pandas.read_csv(fname)
        pred = [0.9 if v else 0.1 for v in df.hasE]
        dfpred = pandas.DataFrame(pred, columns=["c1"])
        s = StringIO()
        dfpred.to_csv(s, index=False)
        sub = s.getvalue()

        db = DatabaseCompetition(":memory:")
        opt = os.path.join(data, "ex_default_options.py")
        db.init_from_options(opt)
        db.connect()

        cid = db.get_cpt_id()
        pid = db.get_player_id()

        db.submit(cid[0], pid[0], sub)
        db.close()

        db.connect()
        subs = list(db.execute("SELECT * FROM submissions"))
        db.close()

        self.assertEqual(len(subs), 2)
        self.assertEqual(
            subs[-1][-2:], ('mean_squared_error', 0.009999999999999997))

    def test_get_competition(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        db = DatabaseCompetition(":memory:")
        data = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        opt = os.path.join(data, "ex_default_options.py")
        db.init_from_options(opt)
        db.connect()
        cpt = db.get_competition(0)
        db.close()
        self.assertIsInstance(cpt, Competition)

    def test_get_results(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        db = DatabaseCompetition(":memory:")
        data = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        opt = os.path.join(data, "ex_default_options.py")
        db.init_from_options(opt)
        db.connect()

        data = os.path.abspath(os.path.join(os.path.dirname(__file__), "data"))
        fname = os.path.join(data, "off_eval_all_Y.txt")
        df = pandas.read_csv(fname)
        pred = [0.9 if v else 0.1 for v in df.hasE]
        dfpred = pandas.DataFrame(pred, columns=["c1"])
        s = StringIO()
        dfpred.to_csv(s, index=False)
        sub = s.getvalue()
        db.submit(0, 0, sub)

        df = db.get_results(0)
        db.close()
        self.assertIsInstance(df, pandas.DataFrame)
        self.assertEqual(df.shape, (1, 9))


if __name__ == "__main__":
    unittest.main()



#-*- coding: utf-8 -*-
"""
@brief      test log(time=33s)
"""

import sys
import os
import unittest


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from tornado.testing import AsyncHTTPTestCase
from src.lightmlboard.appml import LightMLBoard
from tornado.escape import json_encode


class TestLocalAppIndex(AsyncHTTPTestCase):

    def get_app(self):
        this = os.path.dirname(__file__)
        config = os.path.join(this, "this_default_options.py")
        return LightMLBoard.make_app(config=config, logged=dict(user='xd', pwd='pwd'))

    def test_local_index(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")
        post = dict(user=['xd'], pwd=['pwd'],
                    _xsrf=['2|a114f78b|3990916811cdda4bbaec096bf1f57923|1507577457'])
        headers = {'Host': 'localhost:8897', 'Connection': 'keep-alive',
                   'Cache-Control': 'max-age=0',
                   'Origin': 'http://localhost:8897', 'Upgrade-Insecure-Requests': '1',
                   'Content-Type': 'application/x-www-form-urlencoded',
                   'User-Agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/61.0.3163.100 Safari/537.36',
                   'Accept': 'text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,image/apng,*/*;q=0.8',
                   'Referer': 'http://localhost:8897/login',
                   'Accept-Encoding': 'gzip, deflate, br',
                   'Accept-Language': 'fr-FR,fr;q=0.8,en-US;q=0.6,en;q=0.4,ja;q=0.2',
                   'Cookie': '_xsrf=2|5e9d6173|c6190790ee444cb345659f930e7cefdb|1507577457; ' +
                   'incorrect="2|1:0|10:1508700922|9:incorrect|4:NA==|74fbcc0d51ee11cb2073009ee093293a1cddfd8a2315de70e12a17a83df5a2b2"'
                   }
        body = json_encode(post)
        response = self.fetch('/login', method='POST',
                              headers=headers, body=body)
        # Does not work due to:  '_xsrf' argument missing from POST
        return
        self.assertEqual(response.code, 200)
        self.assertNotIn(b"Identification...", response.body)


if __name__ == "__main__":
    unittest.main()



"""
@brief      test log(time=0s)
"""

import sys
import os
import unittest


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    if "PYQUICKHELPER" in os.environ and len(os.environ["PYQUICKHELPER"]) > 0:
        sys.path.append(os.environ["PYQUICKHELPER"])
    import pyquickhelper as skip_


from pyquickhelper.loghelper import fLOG
from pyquickhelper.filehelper import explore_folder_iterfile
from pyquickhelper.pycode import ExtTestCase


class TestConvertNotebooks(ExtTestCase):

    def test_convert_notebooks(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        try:
            import jyquickhelper as skip___
            rem = None
        except ImportError:
            p = os.path.dirname(src.__file__)
            fLOG("add path", p)
            rem = len(sys.path) - 1
            sys.path.append(p)
        from pyquickhelper.ipythonhelper import upgrade_notebook, remove_execution_number
        if rem:
            del sys.path[rem]

        fold = os.path.abspath(os.path.dirname(__file__))
        fold2 = os.path.normpath(
            os.path.join(fold, "..", "..", "_doc", "notebooks"))
        for nbf in explore_folder_iterfile(fold2, pattern=".*[.]ipynb"):
            t = upgrade_notebook(nbf)
            if t:
                fLOG("modified", nbf)
            # remove numbers
            remove_execution_number(nbf, nbf)

        fold2 = os.path.normpath(os.path.join(fold, "..", "..", "_unittests"))
        for nbf in explore_folder_iterfile(fold2, pattern=".*[.]ipynb"):
            t = upgrade_notebook(nbf)
            if t:
                fLOG("modified", nbf)


if __name__ == "__main__":
    unittest.main()



"""
@file
@brief Manages a sqlite3 database.
"""
import numpy
import sqlite3
import datetime
import decimal


class DBException(Exception):
    """
    Exception raised by class @see cl Database.
    """
    pass


class Database:
    """
    Common functions about sqlite3.
    """

    _field_option = ["PRIMARYKEY", "AUTOINCREMENT", "AUTOFILL"]

    def __init__(self, dbfile):
        """
        @param      dbfile      filename or ``:memory:``
        """
        self._sql_file = dbfile
        self._connection = None

    def get_file(self):
        """
        Returns the file name.
        """
        return self._sql_file

    def _check_connection(self):
        """
        Check the SQL connection.
        """
        if self._connection is None:
            message = "Use connect method before doing operation on this database."
            raise Exception(message)

    def _is_memory(self):
        """
        Tells if the database takes place in memory (``:memory:``).
        """
        return self._sql_file == ":memory:"

    def connect(self):
        """
        Opens a connection to the database.
        """
        if self._is_memory():
            if self._connection is None:
                self._connection = sqlite3.connect(self._sql_file)
        elif self._connection is not None:
            raise Exception("A previous connection was not closed.")
        else:
            self._connection = sqlite3.connect(self._sql_file)

    def close(self):
        """
        Close the database.
        """
        self._check_connection()
        if self._is_memory():
            # We should not close, otherwise, we lose the data.
            # self._connection = None
            pass
        else:
            self._connection.close()
            self._connection = None

    def commit(self):
        """
        Call this function after any insert request.
        """
        self._check_connection()
        self._connection.commit()

    def execute(self, request):
        """
        Open a cursor with a query and return it to the user.

        @param      request         SQL request
        @return                     cursor
        """
        # classic ways
        self._check_connection()
        cur = self._connection.cursor()
        try:
            cur.execute(request)
        except Exception as e:
            raise DBException(
                "Unable to execute a SQL request (1) (file '%s')" %
                self.get_file(), e, request) from e
        return cur

    def get_table_list(self):
        """
        Returns the list of tables.

        @return                         the table list
        """
        self._check_connection()
        request = """   SELECT name
                        FROM (SELECT * FROM sqlite_master UNION ALL SELECT * FROM sqlite_temp_master) AS temptbl
                        WHERE type in('table','temp') AND name != 'sqlite_sequence' ORDER BY name;"""

        select = self._connection.execute(request)
        res = []
        for el in select:
            res.append(el[0])
        return res

    def create_table(self, table, columns, temporary=False):
        """
        Creates a table.

        @param      table           table name
        @param      columns         columns definition, dictionary ``{ key:(column_name,python_type) }``
                                    if ``PRIMARYKEY`` is added, the key is considered as the primary key.
                                    Example::

                                        columns = { -1:("key", int, "PRIMARYKEY", "AUTOINCREMENT"),
                                                                0:("name",str), 1:("number", float) }

        @param      temporary       if True the table is temporary
        @return                     cursor
        """
        if table == "sqlite_sequence":
            raise DBException("Unable to create a table named 'sql_sequence'.")

        tables = self.get_table_list()
        if table in tables:
            raise DBException("Tables '{0}' is already present.".format(table))

        if isinstance(columns, list):
            columns = {i: v for i, v in enumerate(columns)}

        if temporary:
            sql = "CREATE TEMPORARY TABLE " + table + "("
        else:
            sql = "CREATE TABLE " + table + "("
        col = []
        for c, val in columns.items():
            if isinstance(val[1], tuple):
                v = val[1][0]
            else:
                v = val[1]

            if v is str:
                col.append(val[0] + " TEXT")
            elif v is int:
                col.append(val[0] + " INTEGER")
            elif v is float:
                col.append(val[0] + " FLOAT")
            elif v is numpy.int64:
                col.append(val[0] + " INTEGER")
            elif v is numpy.float64:
                col.append(val[0] + " FLOAT")
            elif v is decimal.Decimal:
                col.append(val[0] + " Decimal")
            elif v is datetime.datetime:
                col.append(val[0] + " DATETIME")
            else:
                raise DBException(
                    "Unable to add column '{0}' ... {1} v={2}".format(c, val, v))

            if "PRIMARYKEY" in val:
                if val[1] != int:
                    raise DBException(
                        "unable to create a primary key on something differont from an integer (%s)" %
                        str(val))
                col[-1] += " PRIMARY KEY"
                if "AUTOINCREMENT" in val:
                    if self.isMSSQL():
                        col[-1] += " IDENTITY(0,1)"
                    else:
                        col[-1] += " AUTOINCREMENT"

        sql += ",\n       ".join(col)
        sql += ");"
        return self.execute(sql)

    def has_rows(self, table):
        """
        Tells if a table has rows.

        @param      table       table name
        @return                 boolean
        """
        res = list(self.execute("SELECT * FROM {0} LIMIT 1".format(table)))
        return len(res) > 0



# -*- coding: utf-8 -*-
"""
@file
@brief Default options for the application.
"""
import os
from .competition import Competition


class LightMLBoardDefaultOptions:
    """
    Default options for the web application.
    """

    lang = "fr"

    debug = True

    logging = True

    title = "Compétition de Machine Learning"

    subtitle = "ENSAE"

    cookie_secret = "bZJc2sWbQLKos6GkHn/VB9oXwQt8S0R0kRvJ5/xJ89E="

    description = """Tableaux de résultats pour la compétition
        de machine learning organisé dans le cadre du cours
        <a href="http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx3/index.html">Python pour un data scientist économiste</a>.
        """.replace("        ", "")

    description2 = """Détails de la compétition :
        <a href="http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx3/questions/competition_2A.html#l-competition-2017-2a">
        Compétition 2017 - additifs alimentaires</a>.
        """.replace("        ", "")

    title_main = "Principe"
    description_main = """..."""

    allowed_users = None

    competitions = [Competition(
        cpt_id=0,
        name="Prédiction de la présence d'additifs",
        link="http://www.xavierdupre.fr/app/ensae_teaching_cs/helpsphinx3/questions/competition_2A.html#competition-2017-additifs-alimentaires",
        expected_values=os.path.join(
            os.path.dirname(__file__), "data", "dummy_prediction.data"),
        metric=["roc_auc_score_micro", "roc_auc_score_macro"],
        description="""Le site OpenFoodFacts recense la composition de milliers de produits.
                        La base de données peut être téléchargée (data). On veut savoir si les additifs ajoutés
                        apparaissent plus fréquemment avec certains produits ou certaines compositions. Une façon
                        est de prédire la présence d'additifs en fonction de toutes les autres variables.
                        Si un modèle de prédiction fait mieux que le hasard, cela signifie que certaines
                        corrélations existent. C'est un problème de classification binaire.""")]



"""
@file
@brief Helpers around static files.
"""
import os
import shutil


def copy_static(dest):
    """
    Copy static files into *dest/static*.
    """
    dst = os.path.join(dest, "static")
    if not os.path.exists(dst):
        os.mkdir(dst)
    this = os.path.dirname(__file__)
    copied = 0
    for name in os.listdir(this):
        if os.path.splitext(name)[-1] in {'.png', '.ico', '.css', '.js'}:
            shutil.copy(os.path.join(this, name), dst)
            copied += 1
    if copied == 0:
        raise ValueError("No file found in '{0}'.".format(this))



"""
@brief      test log(time=0s)
"""

import sys
import os
import unittest
import re


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from src.lightmlboard import __version__


class TestVersion (unittest.TestCase):

    def test_version(self):
        setup = os.path.join(
            os.path.split(__file__)[0],
            "..",
            "..",
            "setup.py")
        with open(setup, "r") as f:
            c = f.read()
        reg = re.compile("sversion *= \\\"(.*)\\\"")

        f = reg.findall(c)
        if len(f) != 1:
            raise Exception("not only one version")
        self.assertEqual(f[0], __version__)


if __name__ == "__main__":
    unittest.main()



"""
@file
@brief Manages a sqlite3 database to store the results.
"""
import datetime
import numpy
import pandas
from uuid import uuid4
from .dbengine import Database
from .options_helpers import read_options, read_users
from .competition import Competition


class DatabaseCompetition(Database):
    """
    Holds the data used for competitions. Tables:

    Competitions

    * cpt_id
    * cpt_name
    * metric
    * datafile
    * description
    * expected_values

    Teams

    * team_id
    * team_name

    Players

    * player_id
    * team_id
    * player_name
    * mail
    * login
    * pwd

    Submission

    * cpt_id
    * player_id
    * date
    * filename
    * metric_value
    """

    def __init__(self, dbfile):
        """
        @param      dbfile      filename or ``:memory:``
        """
        Database.__init__(self, dbfile)
        self._init()

    def _init(self):
        """
        Creates the tables if not present.
        """
        self.connect()
        tables = self.get_table_list()
        adds = dict(competitions=DatabaseCompetition._col_competitions,
                    teams=DatabaseCompetition._col_teams,
                    players=DatabaseCompetition._col_players,
                    submissions=DatabaseCompetition._col_submissions)
        for k, v in adds.items():
            if k not in tables:
                self.create_table(k, v())
        self.commit()
        self.close()

    def init_from_options(self, filename):
        """
        Initializes the database. It skips a table if
        it exists.

        @param      filename        filename
        """
        opt = read_options(filename)
        if opt is None:
            raise ValueError("No option in '{0}'.".format(filename))
        users = read_users(opt["allowed_users"])
        key = "tmpl_competitions" if "tmpl_competitions" in opt else "competitions"
        competitions = [(d if isinstance(d, Competition)
                         else Competition(**d)) for d in opt[key]]

        if not self.has_rows("teams"):
            teams = map(lambda x: x[1]['team'], users.items())
            tdf = pandas.DataFrame({"team_name": list(teams)})
            tdf.reset_index(drop=False, inplace=True)
            tdf.columns = ["team_id", "team_name"]
            tdf.to_sql("teams", self.Connection,
                       if_exists="append", index=False)

        if not self.has_rows("players"):
            players = list(map(lambda x: x[1], users.items()))
            pdf = pandas.DataFrame(players)
            pdf.reset_index(drop=False, inplace=True)
            tdf = self.to_df("teams")
            pdf["player_name"] = pdf["name"]
            pdf["player_id"] = pdf["index"]
            pdf = pdf.merge(tdf, left_on="team", right_on="team_name")
            pdf = pdf.drop(["name", "team_name", "index", "team"], axis=1)
            pdf.to_sql("players", self.Connection,
                       if_exists="append", index=False)

        if not self.has_rows("competitions"):
            pdf = pandas.DataFrame(Competition.to_records(competitions))
            if "cpt_id" in pdf.columns:
                pdf.reset_index(drop=True, inplace=True)
            else:
                pdf.reset_index(drop=False, inplace=True)
                # tdf = self.to_df("competitions")
                pdf["cpt_id"] = pdf["index"]
                pdf = pdf.drop("index", axis=1)
            pdf.to_sql("competitions", self.Connection,
                       if_exists="append", index=False)

        if not self.has_rows("submissions"):
            pdf = DatabaseCompetition._dummy_submissions()
            pdf.reset_index(drop=True, inplace=True)
            pdf.to_sql("submissions", self.Connection,
                       if_exists="append", index=False)

        df = self.to_df("players")
        logins = set(df['login'])
        if None in logins:
            raise ValueError("One login is wrong: {0}".format(logins))

    def get_competitions(self):
        """
        Returns the list of competitions as list of ``(cpt_id, cpt_name)``.
        """
        return self.execute("SELECT cpt_id, cpt_name FROM competitions")

    def to_df(self, table):
        """
        Returns the content of a table as a dataframe.
        """
        return pandas.read_sql("SELECT * FROM {0}".format(table), self.Connection)

    @property
    def Connection(self):
        """
        Returns the connexion.
        """
        self._check_connection()
        return self._connection

    @staticmethod
    def _col_competitions():
        return [('cpt_id', int), ('link', str), ('cpt_name', str), ('description', str),
                ('metric', str), ('datafile', str), ('expected_values', str)]

    @staticmethod
    def _col_teams():
        return [('team_id', int), ('team_name', str)]

    @staticmethod
    def _col_players():
        return [('player_id', int), ('team_id', int), ('metric', str), ('player_name', str),
                ('mail', str), ('login', str), ('pwd', str)]

    @staticmethod
    def _col_submissions():
        return [('sub_id', str), ('cpt_id', int), ('player_id', int), ('date', str),
                ('data', str), ('metric', str), ('metric_value', float)]

    @staticmethod
    def _dummy_submissions():
        return pandas.DataFrame([dict(sub_id=str(uuid4()), cpt_id=-1, player_id=-1, date=datetime.datetime.now(),
                                      data='', metric='rse', metric_value=numpy.nan)])

    def get_cpt_id(self):
        """
        Returns the list of competation id.
        """
        return list(_[0] for _ in self.execute("SELECT cpt_id FROM competitions"))

    def get_player_id(self):
        """
        Returns the list of competation id.
        """
        return list(_[0] for _ in self.execute("SELECT player_id FROM players"))

    def submit(self, cpt_id, player_id, data, date=datetime.datetime.now()):
        """
        Adds a submission to the database.

        @param      cpt_id          competition id
        @param      player_id       player who did the submission
        @param      data            data of the submission

        The function computes the metric associated to the submission.
        """
        if not isinstance(data, str):
            raise TypeError("data must be str not {0}".format(type(data)))
        cp = list(self.execute(
            "SELECT cpt_id, metric, expected_values FROM competitions WHERE cpt_id={0}".format(cpt_id)))
        if len(cp) == 0:
            raise ValueError("Unable to find cpt_id={0} in\n{1}".format(
                cpt_id, self.get_cpt_id()))
        pid = list(self.execute(
            "SELECT player_id FROM players WHERE player_id={0}".format(player_id)))
        if len(pid) == 0:
            raise ValueError("Unable to find player_id={0} in\n{1}".format(
                player_id, self.get_player_id()))
        metrics = [_[1:] for _ in cp]

        sub = []
        for met, exp in metrics:
            cp = Competition(cpt_id=0, link='', name='', description='',
                             metric=met, expected_values=exp)
            dres = cp.evaluate(data)
            res = dres[met]
            if not isinstance(res, float):
                res = float(res)
            rec = dict(sub_id=str(uuid4()), cpt_id=cpt_id, player_id=player_id,
                       data=data, metric=met, metric_value=res)
            sub.append(rec)

        df = pandas.DataFrame(sub)
        df.to_sql("submissions", self._connection,
                  if_exists="append", index=False)
        self.commit()

    def get_competition(self, cpt_id):
        """
        Retrieves a competition.

        @param      cpt_id  competition id
        @return             @see cl Competition
        """
        res = list(self.execute(
            "SELECT * FROM competitions WHERE cpt_id=={0}".format(cpt_id)))
        if len(res) == 0:
            raise KeyError("No competition for cpt_id=={0}".format(cpt_id))
        if len(res) != 1:
            raise KeyError(
                "Too many competitions for cpt_id=={0}".format(cpt_id))
        args = list(res[0])
        return Competition(*args)

    def get_results(self, cpt_id):
        """
        Retrieves the results of a competition.

        @param      cpt_id  competition id
        @return             a data frame
        """
        res = pandas.read_sql("""SELECT A.*, B.player_name AS player_name, C.team_name
                                 FROM submissions AS A
                                 INNER JOIN players AS B ON A.player_id == B.player_id
                                 INNER JOIN teams AS C ON B.team_id == C.team_id
                                 WHERE cpt_id == {0}
                                 """.format(cpt_id), self.Connection)
        return res



import os


class TestExAppOptions:

    allowed_users = os.path.join(os.path.dirname(__file__), "users.txt")

    competitions = [dict(cpt_id=0,
                         name="compet1",
                         link="http://...",
                         expected_values=os.path.join(os.path.dirname(
                             __file__), "off_eval_all_Y.txt"),
                         description="desc",
                         metric="mean_squared_error")]



"""
@brief      test log(time=0s)
"""

import sys
import os
import unittest
import warnings


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src",)))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import check_pep8, ExtTestCase


class TestFlake8(ExtTestCase):

    def test_flake8_src(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        if sys.version_info[0] == 2 or "Anaconda" in sys.executable \
                or "condavir" in sys.executable:
            warnings.warn(
                "skipping test_flake8 because of Python 2 or " + sys.executable)
            return

        thi = os.path.abspath(os.path.dirname(__file__))
        src_ = os.path.normpath(os.path.join(thi, "..", "..", "src"))
        check_pep8(src_, neg_filter="__init__.py", fLOG=fLOG)

    def test_flake8_test(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        if sys.version_info[0] == 2 or "Anaconda" in sys.executable \
                or "condavir" in sys.executable:
            warnings.warn(
                "skipping test_flake8 because of Python 2 or " + sys.executable)
            return

        thi = os.path.abspath(os.path.dirname(__file__))
        test = os.path.normpath(os.path.join(thi, "..", ))
        check_pep8(test, fLOG=fLOG, neg_filter="temp_.*",
                   skip=["src' imported but unused",
                         "skip_' imported but unused",
                         "skip__' imported but unused",
                         "skip___' imported but unused",
                         ])


if __name__ == "__main__":
    unittest.main()



"""
@file
@brief Defines helpers used in different places of the module.
"""
import importlib
import logging
import os
import sys
import pandas


def read_options(config):
    """
    Reads configuration from a file or a dictionary.
    """
    config_options = {}
    if config is not None:
        if isinstance(config, dict):
            config_options.update(config)
        elif isinstance(config, str) and os.path.exists(config):
            app_log = logging.getLogger("tornado.application")
            app_log.info("[LightMLBoard] read file '{0}'".format(config))
            obj = None
            fname = os.path.splitext(config)[0]
            path, name = os.path.split(fname)
            if path:
                sys.path.append(path)
                mod = importlib.import_module(name)
                del sys.path[-1]
            else:
                mod = importlib.import_module(name)
            for k, v in mod.__dict__.items():
                if isinstance(v, pandas.DataFrame.__class__):
                    obj = v
                    break
            if obj is None:
                raise ValueError(
                    "Unable to read configuration '{0}'.".format(config))
            for k, v in obj.__dict__.items():
                if not k.startswith('_'):
                    config_options[k] = v
        else:
            raise ValueError(
                "Unable to interpret config\ncwd: '{0}'\n{1}".format(os.getcwd(), config))
    return config_options


def read_users(filename):
    """
    Reads users definition.
    """
    df = pandas.read_csv(filename)
    cols = list(sorted(df.columns))
    df = df[cols]
    exp = "login,mail,name,pwd,team".split(",")
    has = list(df.columns)
    if exp != has:
        raise ValueError(
            "Users should be defined in CSV file with columns: {0}, not {1}".format(exp, has))
    users = {}
    for i in range(0, df.shape[0]):
        name, login, mail, pwd, team = (df.loc[i, 'name'], df.loc[i, 'login'],
                                        df.loc[i, 'mail'], df.loc[i, 'pwd'], df.loc[i, 'team'])
        if login in users:
            raise ValueError("Duplicated user: '{0}'.".format(login))
        users[login] = dict(mail=mail, pwd=pwd, team=team,
                            name=name, login=login)
    return users



"""
@file
@brief Metrics about regressions.
"""
import io
import numpy
import pandas
from sklearn.metrics import roc_auc_score


def is_vector(a):
    """
    Tells if an array is a vector.
    """
    return len(a.shape) == 1 or a.shape[1] == 1


def reshape(exp, val):
    """
    Reshape the expected values and predictions.
    """
    if isinstance(val, list):
        val = numpy.array(val)
    if isinstance(exp, list):
        exp = numpy.array(exp)
    if isinstance(val, pandas.DataFrame):
        val = val.as_matrix()
    if isinstance(exp, pandas.DataFrame):
        exp = exp.as_matrix()
    if not isinstance(val, numpy.ndarray):
        raise TypeError("val is {0} not an array".format(type(val)))
    if not isinstance(exp, numpy.ndarray):
        raise TypeError("exp is {0} not an array".format(type(exp)))
    if is_vector(exp) != is_vector(val):
        if not is_vector(val) and is_vector(exp):
            exp_ = exp
            exp = numpy.zeros((val.shape))
            for i, v in enumerate(exp_.ravel()):
                exp[i, int(v)] = 1
        else:
            exp = exp.ravel()
            val = val.ravel()
    elif is_vector(exp) and is_vector(val):
        exp = exp.ravel()
        val = val.ravel()

    if len(exp.shape) == 2 and exp.shape[1] == 1:
        raise ValueError("exp has two dimensions but one column")
    if len(val.shape) == 2 and val.shape[1] == 1:
        raise ValueError("val has two dimensions but one column")
    return exp, val


def roc_auc_score_micro(exp, val):
    """
    Computes `roc_auc_score <http://scikit-learn.org/stable/modules/generated/sklearn.metrics.roc_auc_score.html>`_
    with *average='micro'*.
    """
    exp, val = reshape(exp, val)
    return roc_auc_score(exp, val, average="micro")


def roc_auc_score_macro(exp, val):
    """
    Computes `roc_auc_score <http://scikit-learn.org/stable/modules/generated/sklearn.metrics.roc_auc_score.html>`_
    with *average='macro'*.
    """
    exp, val = reshape(exp, val)
    return roc_auc_score(exp, val, average="macro")


def multi_label_jaccard(exp, val):
    """
    Applies to a multi-label classification problem.
    Computes the average Jaccard index between two sequences
    of sets of labels
    (see `Multi-label classification <https://en.wikipedia.org/wiki/Multi-label_classification>`_).

    @param      exp     list of tuple or list of set or filename or streams (comma separated values) or dict
    @param      val     list of tuple or list of set or filename or streams (comma separated values) or dict
    @return             score

    .. math::

        E = \\frac{1}{n} \\sum_{i=1}^n \\frac{|C_i \\cap P_i|}{|C_i \\cup P_i|}


    """
    def to_set(v):
        if isinstance(v, set):
            return v
        elif isinstance(v, str):
            return set(v.split(','))
        else:
            return set(v)

    if isinstance(exp, (str, io.StringIO)) and isinstance(val, (str, io.StringIO)):
        # Files or streams.
        d1 = pandas.read_csv(exp, header=None, sep=";")
        d2 = pandas.read_csv(val, header=None, sep=";")
        dd1 = {}
        for k, v in d1.itertuples(name=None, index=False):
            if k in dd1:
                raise KeyError("Key '{}' already present".format(k))
            dd1[k] = v
        dd2 = {}
        for k, v in d2.itertuples(name=None, index=False):
            if k in dd2:
                raise KeyError("Key '{}' already present".format(k))
            dd2[k] = v
        return multi_label_jaccard(dd1, dd2)
    elif isinstance(exp, dict) and isinstance(val, dict):
        if len(exp) != len(val):
            raise ValueError(
                "Dimension mismatch {0} != {1}".format(len(exp), len(val)))
        r = 0.0
        for k, e in exp.items():
            if k in val:
                v = val[k]
                es = to_set(e)
                vs = to_set(v)
                r += float(len(es & vs)) / len(es.union(vs))
            else:
                raise ValueError("Missing key in prediction {0}".format(k))
        return r / len(exp)
    elif isinstance(exp, list) and isinstance(val, list):
        if len(exp) != len(val):
            raise ValueError(
                "Dimension mismatch {0} != {1}".format(len(exp), len(val)))

        r = 0.0
        for e, v in zip(exp, val):
            es = to_set(e)
            vs = to_set(v)
            r += float(len(es & vs)) / len(es.union(vs))
        return r / len(exp)
    else:
        raise TypeError(
            "Inconsisten types {0} != {1}".format(type(exp), type(val)))



"""
@brief      test tree node (time=50s)
"""

import sys
import os
import unittest

try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import get_temp_folder, ExtTestCase

if sys.version_info[0] == 2:
    from codecs import open


class TestReadme(ExtTestCase):

    def test_venv_docutils08_readme(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        fold = os.path.dirname(os.path.abspath(__file__))
        readme = os.path.join(fold, "..", "..", "README.rst")
        self.assertTrue(os.path.exists(readme))
        with open(readme, "r", encoding="utf8") as f:
            content = f.read()

        self.assertTrue(len(content) > 0)
        temp = get_temp_folder(__file__, "temp_readme")

        if __name__ != "__main__":
            # does not work from a virtual environment
            return

        from pyquickhelper.pycode import check_readme_syntax

        check_readme_syntax(readme, folder=temp, fLOG=fLOG)


if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
import os
from setuptools import setup, Extension
from setuptools import find_packages

#########
# settings
#########

project_var_name = "lightmlboard"
sversion = "0.1"
versionPython = "%s.%s" % (sys.version_info.major, sys.version_info.minor)
path = "Lib/site-packages/" + project_var_name
readme = 'README.rst'
history = "HISTORY.rst"
requirements = None

KEYWORDS = project_var_name + ', Xavier Dupré'
DESCRIPTION = """Light web server serving pages on a leaderboard for machine learned competitions"""


CLASSIFIERS = [
    'Programming Language :: Python :: 3',
    'Intended Audience :: Developers',
    'Topic :: Scientific/Engineering',
    'Topic :: Education',
    'License :: OSI Approved :: MIT License',
    'Development Status :: 5 - Production/Stable'
]


#######
# data
#######


packages = find_packages('src', exclude='src')
package_dir = {k: "src/" + k.replace(".", "/") for k in packages}
package_data = {
    project_var_name + ".static": ["*.js", "*.css", "*.ico", "*.png", "*.gif"],
    project_var_name + ".templates": ["*.html"],
    project_var_name + ".data": ["*.txt"],
}

############
# functions
############


def is_local():
    file = os.path.abspath(__file__).replace("\\", "/").lower()
    if "/temp/" in file and "pip-" in file:
        return False
    if \
       "bdist_msi" in sys.argv or \
       "build27" in sys.argv or \
       "build_script" in sys.argv or \
       "build_sphinx" in sys.argv or \
       "build_ext" in sys.argv or \
       "bdist_wheel" in sys.argv or \
       "bdist_wininst" in sys.argv or \
       "clean_pyd" in sys.argv or \
       "clean_space" in sys.argv or \
       "copy27" in sys.argv or \
       "copy_dist" in sys.argv or \
       "local_pypi" in sys.argv or \
       "notebook" in sys.argv or \
       "publish" in sys.argv or \
       "publish_doc" in sys.argv or \
       "register" in sys.argv or \
       "unittests" in sys.argv or \
       "unittests_LONG" in sys.argv or \
       "unittests_SKIP" in sys.argv or \
       "unittests_GUI" in sys.argv or \
       "run27" in sys.argv or \
       "sdist" in sys.argv or \
       "setupdep" in sys.argv or \
       "test_local_pypi" in sys.argv or \
       "upload_docs" in sys.argv or \
       "setup_hook" in sys.argv or \
       "copy_sphinx" in sys.argv or \
       "write_version" in sys.argv:
        try:
            import_pyquickhelper()
        except ImportError:
            return False
        return True
    else:
        return False


def import_pyquickhelper():
    try:
        import pyquickhelper
    except ImportError:
        sys.path.append(
            os.path.normpath(
                os.path.abspath(
                    os.path.join(
                        os.path.dirname(__file__),
                        "..",
                        "pyquickhelper",
                        "src"))))
        try:
            import pyquickhelper
        except ImportError as e:
            message = "module pyquickhelper is needed to build the documentation ({0}), not found in path {1}".format(
                sys.executable,
                sys.path[
                    -1])
            raise ImportError(message) from e
    return pyquickhelper


def verbose():
    print("---------------------------------")
    print("package_dir =", package_dir)
    print("packages    =", packages)
    print("package_data=", package_data)
    print("current     =", os.path.abspath(os.getcwd()))
    print("---------------------------------")

##########
# version
##########


if is_local() and "--help" not in sys.argv and "--help-commands" not in sys.argv:
    def write_version():
        pyquickhelper = import_pyquickhelper()
        from pyquickhelper.pycode import write_version_for_setup
        return write_version_for_setup(__file__)

    if sys.version_info[0] != 2:
        write_version()

    versiontxt = os.path.join(os.path.dirname(__file__), "version.txt")
    if os.path.exists(versiontxt):
        with open(versiontxt, "r") as f:
            lines = f.readlines()
        subversion = "." + lines[0].strip("\r\n ")
        if subversion == ".0":
            raise Exception("Subversion is wrong: '{0}'.".format(subversion))
    else:
        raise FileNotFoundError(versiontxt)
else:
    # when the module is installed, no commit number is displayed
    subversion = ""

if "upload" in sys.argv and not subversion:
    # avoid uploading with a wrong subversion number
    try:
        import pyquickhelper
        pyq = True
    except ImportError:
        pyq = False
    raise Exception(
        "subversion is empty, cannot upload, is_local()={0}, pyquickhelper={1}".format(is_local(), pyq))

##############
# common part
##############

if os.path.exists(readme):
    if sys.version_info[0] == 2:
        from codecs import open
    with open(readme, "r", encoding='utf-8-sig') as f:
        long_description = f.read()
else:
    long_description = ""
if os.path.exists(history):
    if sys.version_info[0] == 2:
        from codecs import open
    with open(history, "r", encoding='utf-8-sig') as f:
        long_description += f.read()

if "--verbose" in sys.argv:
    verbose()

if is_local():
    pyquickhelper = import_pyquickhelper()
    logging_function = pyquickhelper.get_fLOG()
    logging_function(OutputPrint=True)
    must_build, run_build_ext = pyquickhelper.get_insetup_functions()

    if must_build():
        out = run_build_ext(__file__)
        print(out)

    if "build_sphinx" in sys.argv and not sys.platform.startswith("win"):
        # There is an issue with matplotlib and notebook gallery on linux
        # _tkinter.TclError: no display name and no $DISPLAY environment variable
        import matplotlib
        matplotlib.use('agg')

    from pyquickhelper.pycode import process_standard_options_for_setup
    r = process_standard_options_for_setup(
        sys.argv, __file__, project_var_name,
        unittest_modules=["pyquickhelper"],
        additional_notebook_path=["pyquickhelper", "jyquickhelper"],
        additional_local_path=["pyquickhelper", "jyquickhelper"],
        requirements=["pyquickhelper", "jyquickhelper"],
        layout=["html"],
        add_htmlhelp=sys.platform.startswith("win"),
        coverage_options=dict(omit=["*exclude*.py"]),
        fLOG=logging_function, covtoken=("92a8ebe9-8c58-4ff2-b5df-ce0540d648f2", "'_UT_36_std' in outfile"))
    if not r and not ({"bdist_msi", "sdist",
                       "bdist_wheel", "publish", "publish_doc", "register",
                       "upload_docs", "bdist_wininst", "build_ext"} & set(sys.argv)):
        raise Exception("unable to interpret command line: " + str(sys.argv))
else:
    r = False

if not r:
    if len(sys.argv) in (1, 2) and sys.argv[-1] in ("--help-commands",):
        pyquickhelper = import_pyquickhelper()
        from pyquickhelper.pycode import process_standard_options_for_setup_help
        process_standard_options_for_setup_help(sys.argv)
    root = os.path.abspath(os.path.dirname(__file__))
    if sys.platform.startswith("win"):
        extra_compile_args = None
    else:
        extra_compile_args = ['-std=c++11']
    setup(
        name=project_var_name,
        version='%s%s' % (sversion, subversion),
        author='Xavier Dupré',
        author_email='xavier.dupre@gmail.com',
        license="MIT",
        url="http://www.xavierdupre.fr/app/%s/helpsphinx/index.html" % project_var_name,
        download_url="https://github.com/sdpython/%s/" % project_var_name,
        description=DESCRIPTION,
        long_description=long_description,
        keywords=KEYWORDS,
        classifiers=CLASSIFIERS,
        packages=packages,
        package_dir=package_dir,
        package_data=package_data,
        #data_files              = data_files,
        install_requires=['numpy', 'pandas', 'tornado'],
        #include_package_data    = True,
    )



#-*- coding: utf-8 -*-
"""
@brief      test log(time=1s)
"""

import sys
import os
import unittest


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import ExtTestCase
from src.lightmlboard.competition import Competition
from src.lightmlboard.default_options import LightMLBoardDefaultOptions


class TestCompetition(ExtTestCase):

    def test_competition(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        vals = LightMLBoardDefaultOptions.competitions
        ds = [v.to_dict() for v in vals]
        vals2 = [Competition(**d) for d in ds]
        ds2 = [v.to_dict() for v in vals2]
        self.assertEqual(ds, ds2)


if __name__ == "__main__":
    unittest.main()



#-*- coding: utf-8 -*-
"""
@brief      test log(time=33s)
"""

import sys
import os
import unittest


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import get_temp_folder
from tornado.testing import AsyncHTTPTestCase
from src.lightmlboard.appml import LightMLBoard
from src.lightmlboard.static import copy_static


class TestLocalAppWaiting(AsyncHTTPTestCase):

    def get_app(self):
        this = os.path.dirname(__file__)
        config = os.path.join(this, "this_default_options.py")
        return LightMLBoard.make_app(config=config, logged=dict(user='xd', pwd='pwd'))

    def test_local_waiting(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")
        response = self.fetch('/upload')
        self.assertEqual(response.code, 200)
        self.assertIn(b"<h2>Uploading</h2>", response.body)
        self.assertIn(b".gif", response.body)
        temp = get_temp_folder(__file__, "temp_local_waiting")
        page = os.path.join(temp, "waiting.html")
        with open(page, "wb") as f:
            f.write(response.body)
        copy_static(temp)


if __name__ == "__main__":
    unittest.main()



#-*- coding: utf-8 -*-
"""
@brief      test log(time=1s)
"""

import sys
import os
import unittest


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import ExtTestCase
from src.lightmlboard.competition import Competition


class TestMetrics(ExtTestCase):

    def test_mse(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        compet = Competition(0, link="/compet", name="compet1",
                             description="description", metric="mse",
                             expected_values=[[0, 1, 2]])
        res = compet.evaluate([[0, 1, 2]])
        self.assertEqual(res, {'mse': 0.0})
        res = compet.evaluate([[0, 4, 2]])
        self.assertEqual(res, {'mse': 3.0})

    def test_mean_squared_error(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        compet = Competition(0, link="/compet", name="compet1",
                             description="description", metric="mean_squared_error",
                             expected_values=[[0, 1, 2]])
        res = compet.evaluate([[0, 1, 2]])
        self.assertEqual(res, {'mean_squared_error': 0.0})
        res = compet.evaluate([[0, 4, 2]])
        self.assertEqual(res, {'mean_squared_error': 3.0})

    def test_roc_auc_score(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        compet = Competition(0, link="/compet", name="compet1",
                             description="description", metric="roc_auc_score_micro",
                             expected_values=[0, 1, 0, 1])
        res = compet.evaluate([[0, 1, 0, 1]])
        self.assertEqual(res, {'roc_auc_score_micro': 1.0})
        res = compet.evaluate(
            [[0.1, 0.9], [0.1, 0.9], [0.1, 0.9], [0.9, 0.1]])
        self.assertEqual(res, {'roc_auc_score_micro': 0.25})
        res = compet.evaluate(
            [[0.1, 0.9], [0.9, 0.1], [0.1, 0.9], [0.9, 0.1]])
        self.assertEqual(res, {'roc_auc_score_micro': 0.0})
        res = compet.evaluate(
            [[0.9, 0.1], [0.1, 0.9], [0.9, 0.1], [0.1, 0.9]])
        self.assertEqual(res, {'roc_auc_score_micro': 1.0})

        compet = Competition(0, link="/compet", name="compet1",
                             description="description", metric="roc_auc_score_macro",
                             expected_values=[0, 1, 0, 1])
        res = compet.evaluate([0, 1, 0, 1])
        self.assertEqual(res, {'roc_auc_score_macro': 1.0})
        res = compet.evaluate(
            [[0.1, 0.9], [0.1, 0.9], [0.1, 0.9], [0.9, 0.1]])
        self.assertEqual(res, {'roc_auc_score_macro': 0.25})

        compet = Competition(0, link="/compet", name="compet1",
                             description="description", metric="roc_auc_score_micro",
                             expected_values=[0, 1, 0, 1])
        res = compet.evaluate([[0.1, 0.9, 0.1, 0.9]])
        self.assertEqual(res, {'roc_auc_score_micro': 1.0})


if __name__ == "__main__":
    unittest.main()



#-*- coding: utf-8 -*-
"""
@brief      test log(time=33s)
"""

import sys
import os
import unittest


try:
    import pyquickhelper as skip_
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..",
                "..",
                "pyquickhelper",
                "src")))
    if path not in sys.path:
        sys.path.append(path)
    import pyquickhelper as skip_


try:
    import src
except ImportError:
    path = os.path.normpath(
        os.path.abspath(
            os.path.join(
                os.path.split(__file__)[0],
                "..",
                "..")))
    if path not in sys.path:
        sys.path.append(path)
    import src

from pyquickhelper.loghelper import fLOG
from pyquickhelper.pycode import get_temp_folder, ExtTestCase
from pyquickhelper.ipythonhelper import execute_notebook_list, execute_notebook_list_finalize_ut
from pyquickhelper.pycode import is_travis_or_appveyor
from pyquickhelper.ipythonhelper import install_python_kernel_for_unittest
import src.lightmlboard


class TestRunNotebooksPython(ExtTestCase):

    def test_run_notebook(self):
        fLOG(
            __file__,
            self._testMethodName,
            OutputPrint=__name__ == "__main__")

        if sys.version_info[0] == 2:
            # notebooks are not converted into python 2.7, so not tested
            return

        kernel_name = None if is_travis_or_appveyor() else install_python_kernel_for_unittest(
            "lightmlboard")

        temp = get_temp_folder(__file__, "temp_run_notebooks")

        # selection of notebooks
        fnb = os.path.normpath(os.path.join(
            os.path.abspath(os.path.dirname(__file__)), "..", "..", "_doc", "notebooks"))
        keepnote = []
        if not os.path.exists(fnb):
            return
        for f in os.listdir(fnb):
            if os.path.splitext(f)[-1] == ".ipynb" and "_long" not in f:
                keepnote.append(os.path.join(fnb, f))
        self.assertGreater(len(keepnote), 0)

        # function to tell that a can be run
        def valid(cell):
            if "open_html_form" in cell:
                return False
            if "open_window_params" in cell:
                return False
            if '<div style="position:absolute' in cell:
                return False
            return True

        # additionnal path to add
        addpaths = [os.path.normpath(os.path.join(
            os.path.abspath(os.path.dirname(__file__)), "..", "..", "src")),
            os.path.normpath(os.path.join(
                os.path.abspath(os.path.dirname(__file__)), "..", "..", "..", "jyquickhelper", "src")),
            os.path.normpath(os.path.join(
                os.path.abspath(os.path.dirname(__file__)), "..", "..", "..", "pyquickhelper", "src"))
        ]

        # run the notebooks
        res = execute_notebook_list(
            temp, keepnote, fLOG=fLOG, valid=valid, additional_path=addpaths, kernel_name=kernel_name)
        execute_notebook_list_finalize_ut(
            res, fLOG=fLOG, dump=src.lightmlboard)


if __name__ == "__main__":
    unittest.main()
